#
# utils.py - API utils
#
# Copyright 2013, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>
#   Ilgiz Islamgulov <ilgizisl@gmail.com>
#
import json
import datetime

from flask import Response

from errors import MalformedJSONError, NoSuchObjectError


class DateTimeJsonEncoder(json.JSONEncoder):
    """
    JSONEncoder subclass that knows how to encode date/time.
    """

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            r = obj.isoformat()
            if obj.microsecond:
                r = r[:23] + r[26:]
            if r.endswith('+00:00'):
                r = r[:-6] + 'Z'
            return r
        elif isinstance(obj, datetime.date):
            return obj.isoformat()
        elif isinstance(obj, datetime.time):
            #determines if a given datetime.datetime is aware.
            if obj.tzinfo is not None and obj.tzinfo.utcoffset(
                    obj) is not None:
                raise ValueError("JSON can't represent timezone-aware times.")
            r = obj.isoformat()
            if obj.microsecond:
                r = r[:12]
            return r
        return super(DateTimeJsonEncoder, self).default(obj)


class JsonResponse(Response):
    default_mimetype = 'application/json'

    def __init__(self, response=None, *args, **kwargs):
        if response is None:
            response = []
        data = json.dumps(response, cls=DateTimeJsonEncoder)
        super(JsonResponse, self).__init__(data, *args, **kwargs)


def json_loads(data):
    try:
        return json.loads(data)
    except (ValueError, TypeError), e:
        raise MalformedJSONError(detail=str(e))


def get_or_404(model, *args, **kwargs):
    rv = model.query.filter_by(*args, **kwargs).first()
    if rv is None:
        raise NoSuchObjectError(obj_type=model.__name__)
    return rv
